@@ -21,11 +21,12 @@ from xonsh.tools import setup_win_unicode_console, print_color, to_bool_or_int
 from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
 from xonsh.codecache import run_script_with_cache, run_code_with_cache
 from xonsh.xonfig import print_welcome_screen
+from xonsh.xontribs import xontribs_load
 from xonsh.lazyimps import pygments, pyghooks
 from xonsh.imphooks import install_import_hooks
 from xonsh.events import events
 from xonsh.environ import xonshrc_context, make_args_env
-from xonsh.xontribs import xontribs_load
+from xonsh.built_ins import XonshSession, load_builtins, load_proxies
 
 
 events.transmogrify("on_post_init", "LoadEvent")
@@ -249,7 +250,7 @@ def _pprint_displayhook(value):
     if isinstance(value, HiddenCommandPipeline):
         builtins._ = value
         return
-    env = builtins.__xonsh_env__
+    env = builtins.__xonsh__.env
     if env.get("PRETTY_PRINT_RESULTS"):
         printed_val = pretty(value)
     else:
@@ -287,7 +288,7 @@ def start_services(shell_kwargs, args):
     events.on_timingprobe.fire(name="post_execer_init")
     # load rc files
     login = shell_kwargs.get("login", True)
-    env = builtins.__xonsh_env__
+    env = builtins.__xonsh__.env
     rc = shell_kwargs.get("rc", None)
     rc = env.get("XONSHRC") if rc is None else rc
     if args.mode != XonshMode.interactive and not args.force_interactive:
@@ -297,7 +298,7 @@ def start_services(shell_kwargs, args):
     xonshrc_context(rcfiles=rc, execer=execer, ctx=ctx, env=env, login=login)
     events.on_post_rc.fire()
     # create shell
-    builtins.__xonsh_shell__ = Shell(execer=execer, **shell_kwargs)
+    builtins.__xonsh__.shell = Shell(execer=execer, **shell_kwargs)
     ctx["__name__"] = "__main__"
     return env
 
@@ -310,7 +311,7 @@ def premain(argv=None):
     setproctitle = get_setproctitle()
     if setproctitle is not None:
         setproctitle(" ".join(["xonsh"] + argv))
-    builtins.__xonsh_ctx__ = {}
+    builtins.__xonsh__ = XonshSession()
     args = parser.parse_args(argv)
     if args.help:
         parser.print_help()
@@ -325,7 +326,7 @@ def premain(argv=None):
         "login": False,
         "scriptcache": args.scriptcache,
         "cacheall": args.cacheall,
-        "ctx": builtins.__xonsh_ctx__,
+        "ctx": builtins.__xonsh__.ctx,
     }
     if args.login:
         shell_kwargs["login"] = True
@@ -413,8 +414,8 @@ def main_xonsh(args):
         signal.signal(signal.SIGTTOU, func_sig_ttin_ttou)
 
     events.on_post_init.fire()
-    env = builtins.__xonsh_env__
-    shell = builtins.__xonsh_shell__
+    env = builtins.__xonsh__.env
+    shell = builtins.__xonsh__.shell
     try:
         if args.mode == XonshMode.interactive:
             # enter the shell
@@ -460,8 +461,7 @@ def postmain(args=None):
     """Teardown for main xonsh entry point, accepts parsed arguments."""
     if ON_WINDOWS:
         setup_win_unicode_console(enable=False)
-    if hasattr(builtins, "__xonsh_shell__"):
-        del builtins.__xonsh_shell__
+    builtins.__xonsh__.shell = None
 
 
 @contextlib.contextmanager
@@ -471,7 +471,7 @@ def main_context(argv=None):
     up the shell.
     """
     args = premain(argv)
-    yield builtins.__xonsh_shell__
+    yield builtins.__xonsh__.shell
     postmain(args)
 
 
@@ -508,15 +508,16 @@ def setup(
     """
     ctx = {} if ctx is None else ctx
     # setup xonsh ctx and execer
-    builtins.__xonsh_ctx__ = ctx
-    builtins.__xonsh_execer__ = Execer(xonsh_ctx=ctx)
-    builtins.__xonsh_shell__ = Shell(
-        builtins.__xonsh_execer__, ctx=ctx, shell_type=shell_type
-    )
-    builtins.__xonsh_env__.update(env)
+    if not hasattr(builtins, "__xonsh__"):
+        execer = Execer(xonsh_ctx=ctx)
+        builtins.__xonsh__ = XonshSession(ctx=ctx, execer=execer)
+        load_builtins(ctx=ctx, execer=execer)
+        load_proxies()
+        builtins.__xonsh__.shell = Shell(execer, ctx=ctx, shell_type=shell_type)
+    builtins.__xonsh__.env.update(env)
     install_import_hooks()
     builtins.aliases.update(aliases)
     if xontribs:
         xontribs_load(xontribs)
-    tp = builtins.__xonsh_commands_cache__.threadable_predictors
+    tp = builtins.__xonsh__.commands_cache.threadable_predictors
     tp.update(threadable_predictors)
